//System-includes
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <stdlib.h>
#include <string.h>

//own includes
#include "main.h"
#include "lcd.h"
#include "i2cmaster.h"
#include "uart.h"
#include "key.h"


//These are the user-defined characters for the LCD (up to 8)
//static const uint8_t usergraphics[] EEMEM =
/*
uint8_t usergraphics[] EEMEM =

{
	 0, 2, 6,14, 6, 2, 0, 0,	//Arrow left
	 0, 8,12,14,12, 8, 0, 0,	//Arrow right
	 0, 0, 4,14,31, 0, 0, 0,	//Arrow up
	 0, 0,31,14, 4, 0, 0, 0,	//Arrow down
	 0, 1, 3,22,28, 8, 0, 0,	//a accept-sign
	 0,27,14, 4,14,27, 0, 0,	//a discard-sign
	 4,14,14,14,31, 0, 4, 0,	//a bell
	 8,20, 8, 3, 4, 4, 3, 0		//the "C" symbol
};
*/

//other strings for the LCD
static const char menu_level0_string[] EEMEM =		"Setup          Reset";
//static const char menu_level1_string[] EEMEM =		"\4     \3     \2      \5";
//static const char menu_level2_string[] EEMEM =		"\4     \3     \2      \1";
//static const char menu_level3_string[] EEMEM =		"\4                  \5";
//static const char desc_condition_string[] EEMEM =	"Sensor EIN oder AUS?";
//static const char desc_output_string[] EEMEM =		" Ausgang? 1-4 (5-8) ";
//static const char desc_alarm_string[] EEMEM =		"  Alarm ausloesen?  ";
//static const char desc_hysteresis_string[] EEMEM =	"  Hysterese in \7?   ";
//static const char desc_address_string[] EEMEM =		"Adresse des Sensors?";
//static const char desc_target_value_string[] EEMEM =	"   Sollwert in \7?   ";
//static const char desc_reset_string1[] EEMEM =		"       RESET        ";
//static const char desc_reset_string2[] EEMEM =		"     AUSLOESEN?     ";
//static const char desc_choose_sensor_string[] EEMEM =	"Sensor waehlen [1-6]";

//char menu_level0_string[] EEMEM =		"Setup          Reset";
char menu_level1_string[] EEMEM =		"\4     \3     \2      \5";
char menu_level2_string[] EEMEM =		"\4     \3     \2      \1";
char menu_level3_string[] EEMEM =		"\4                  \5";
char desc_condition_string[] EEMEM =	"Sensor EIN oder AUS?";
char desc_output_string[] EEMEM =		" Ausgang? 1-4 (5-8) ";
char desc_alarm_string[] EEMEM =		"  Alarm ausloesen?  ";
char desc_hysteresis_string[] EEMEM =	"  Hysterese in \7?   ";
char desc_address_string[] EEMEM =		"Adresse des Sensors?";
char desc_target_value_string[] EEMEM =	"   Sollwert in \7?   ";
char desc_reset_string1[] EEMEM =		"       RESET        ";
char desc_reset_string2[] EEMEM =		"     AUSLOESEN?     ";
char desc_choose_sensor_string[] EEMEM =	"Sensor waehlen [1-6]";

/*
//Use this funtion once to load the user defined graphics into the LCD
void load_usergraphics(void)
{
	uint8_t i;
	lcd_command(_BV(LCD_CGRAM));	//set address of CGRAM
	for(i=0; i<64; i++)		//loop through the values
		lcd_data(eeprom_read_byte(&usergraphics[i]));
                
}               
*/ 
	            
void write_menus(uint8_t iScreen)
{
		uint8_t i;

		lcd_gotoxy(0,0);
		switch(iScreen)
		{
			case 0:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_condition_string[i]));
				break;
			}
			case 1:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_output_string[i]));
				break;
			}
			case 2:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_alarm_string[i]));
				break;
			}
			case 3:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_hysteresis_string[i]));
				break;
			}
			case 4:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_address_string[i]));
				break;
			}
			case 5:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_target_value_string[i]));
				break;
			}

			case 6:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_choose_sensor_string[i]));
				break;
			}
		}

		lcd_gotoxy(0,3);
		switch(iScreen)
		{
			case 10:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&menu_level0_string[i]));
				break;
			}
			case 11:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&menu_level1_string[i]));
				break;
			}
			case 12:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&menu_level2_string[i]));
				break;
			}
			case 13:
			{
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&menu_level3_string[i]));

				lcd_gotoxy(0,0);
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_reset_string1[i]));

				lcd_gotoxy(0,1);
				for(i=0; i<20; i++)
					lcd_putc(eeprom_read_byte(&desc_reset_string2[i]));

				break;
			}
		}
}

//This routine takes the X- and Y-position of the LCD, the Number of the Sensor
//and a pointer to the sensor itself and print it on the LCD.
void write_to_position(uint8_t posx, uint8_t posy, uint8_t sensor_number, tSensor *sensor)
{               
	char sOutString[20], sTemp[10];
	uint8_t iLen=0;

	if((sensor->condition) == false)		//Check, if sensor is marked as "off"
	{       
		lcd_gotoxy(posy,posx);			//set new position on the lcd
		lcd_puts("         ");			//output blanks
		return;					//and return
	}       
	
	itoa((sensor_number),sOutString,10);		//convert the sensor-number
	strcat(sOutString, " ");			//add a whitespace
	
	if ((sensor->actual_value) == SENSOR_ERROR)	//if the value is set to an error-value
		strcat(sOutString, "ERROR  ");		//print out the word "ERROR"
	else						//otherwise continue
	{
		itoa((sensor->actual_value),sTemp,10);	//convert the value to a string
		iLen = strlen(sTemp);			//and calculate the length
	
		for(iLen; iLen < 3; iLen++)		//add a number of whitespaces before (right-adjust the value)
			strcat(sOutString, " ");
		
		strcat(sOutString, sTemp);		//convert the temperature
		strcat(sOutString, "\7");		//and add the "C"-sign
		
		if((sensor->output_state) == false)	//if the state of the output is "off"
			strcat(sOutString, " \5");	//add an "X"-sign
		else
			strcat(sOutString, " \4");	//or add an "checked"-sign
	
		if((sensor->alarm) == false)		//if the alarm is not set for this output
			strcat(sOutString, " ");	//add a whitespace
		else					//or
			strcat(sOutString, "\6");	//add the "bell"-sign
	}
				
	lcd_gotoxy(posy,posx);				//set new position on the lcd
	lcd_puts(sOutString);				//and finally output the whole string
}


//This routine prints the first menu-level to the screen and returns
//either the selected sensor as integer-value or zero
uint8_t write_choose_sensor(void)
{
	uint8_t iSensor=1;				//We always start with sensor 1

	lcd_clrscr();					//clean the display
	write_menus(6);					//Write the description to the top of the display
	write_menus(11);				//and the menu to the bottom
	lcd_gotoxy(5,2);				//set new position on the lcd
	lcd_puts("Sensor: ");				//and print the string

	while(true)					//now run into a loop, print the values and check keys
	{
		wdt_reset();				//reset watchdog
		
		if(get_key_press(1<<BUTTON2))		//If button 2 is pressed
		{
			if(iSensor>1)			//and the value is not below 1
				iSensor--;		//decrease the sensor-number
		}

		if(get_key_press(1<<BUTTON3))		//If button 3 is pressed
		{
			if(iSensor<6)			//and the value is not higher than 6
				iSensor++;		//increase the sensor-number
		}
	
		if(get_key_press(1<<BUTTON1))		//If button 1 is pressed
		{
			lcd_clrscr();			//clean up the lcd
			return iSensor;			//break the loop and return the number of the sensor
		}

		if(get_key_press(1<<BUTTON4))		//If button 1 is pressed
		{
			lcd_clrscr();			//clean up the lcd
			return 0;			//break the loop and return zero
		}
	
		lcd_gotoxy(13,2);			//if we still here, set new position on the lcd
		lcd_putc(iSensor+48);			//and print the new value

	}
}

//This routine handles the complete settings for all sensors
//it only takes a pointer to the sensor itself
void write_settings_menu(tSensor *sensor)
{
	uint8_t i, iScreen=0, iScreenWritten=true;
	char sTemp[6];

	lcd_clrscr();					//clean the display
	lcd_gotoxy(0,3);				//and print the menu to the bottom
	for(i=0; i<20; i++)
		lcd_putc(eeprom_read_byte(&menu_level2_string[i]));

	while(true)					//now run into a loop
	{
		wdt_reset();					//reset watchdog
		
		if(iScreenWritten != false)		//check, if we had to update the screen
		{
			write_menus(iScreen);		//print the description, based on screen-number
			
			lcd_gotoxy(0,2);
			lcd_puts("                    ");	//Clean up the 3rd line of the display
			lcd_gotoxy(8,2);
			switch(iScreen)			//cycle through the screens
			{
				case 0:			//we need to print the condition of the sensor
				{
					if((sensor->condition) == 0)
						lcd_puts("AUS");
					else
						lcd_puts("EIN");
					break;
				}
				case 1:			//we need to print the number of the selected output
				{
					lcd_putc((sensor->output)+49);
					break;
				}
				case 2:			//we need to print the alarm-status
				{
					if((sensor->alarm) == 0)
						lcd_puts("AUS");
					else
						lcd_puts("EIN");
					break;				
				}
				case 3:			//we need to print the hysteresis
				{
					lcd_putc((sensor->hysteresis)+48);				
					break;
				}
				case 4:			//we need to print the sensor-address
				{
					lcd_puts(itoa((sensor->i2c_address),sTemp,10));
					break;
				}
				case 5:			//we need to print the target-value
				{
					lcd_puts(itoa((sensor->target_value),sTemp,10));
					break;
				}
			}

			iScreenWritten = 0;		//the screen has been updated
		}

		if(get_key_press(1<<BUTTON2)|get_key_rpt(1<<BUTTON2))	//if button2 is pressed
		{

			switch(iScreen)			//cycle through the screens
			{
				case 0:			//switch the sensor off
				{
					(sensor->condition) = 0;
					break;
				}
				case 1:			//decrease output-number
				{
					if((sensor->output) > 0)
						(sensor->output) -= 1;
					break;
				}
				case 2:			//switch alarm off
				{
					(sensor->alarm) = 0;
					break;				
				}
				case 3:			//decrease hysteresis
				{
					if((sensor->hysteresis) > 1)
						(sensor->hysteresis) -= 1;
					break;
				}
				case 4:			//decrease the i2c-address
				{
					if((sensor->i2c_address) > 1)
						(sensor->i2c_address) -= 1;
					break;
				}
				case 5:			//decrease the target-value
				{
					if((sensor->target_value) > MIN_VALUE)
						(sensor->target_value) -= 1;
					break;
				}
			}
			iScreenWritten = true;		//the screen needs an update
		}

		if(get_key_press(1<<BUTTON3)|get_key_rpt(1<<BUTTON3))	//if button3 is pressed
		{
			switch(iScreen)
			{
				case 0:			//switch the sensor on
				{
					(sensor->condition) = 1;
					break;
				}
				case 1:			//increase output-number
				{
					if((sensor->output) < 7)
						(sensor->output) += 1;
					break;
				}
				case 2:			//switch alarm on
				{
					(sensor->alarm) = 1;
					break;				
				}
				case 3:			//increase hysteresis
				{
					if((sensor->hysteresis) < 9)
						(sensor->hysteresis) += 1;
					break;
				}
				case 4:			//increase the i2c-address
				{
					if((sensor->i2c_address) < 255)
						(sensor->i2c_address) += 1;
					break;
				}
				case 5:			//increase the target-value
				{
					if((sensor->target_value) < MAX_VALUE)
						(sensor->target_value) += 1;
					break;
				}
			}
			iScreenWritten = true;		//the screen needs an update
		}

		if(get_key_press(1<<BUTTON1))		//If button 1 is pressed
		{
			lcd_clrscr();			//clean up the lcd
			return;				//and return
		}

		if(get_key_press(1<<BUTTON4))		//If button 4 is pressed,
		{
			if(iScreen < 5)			//and the screen-number is not larger than 5
				iScreen++;		//switch to next screen
			else				
				iScreen = 0;		//else reset screen-counter
			iScreenWritten = true;		//the screen needs an update
		}
	}	
}
